/* -*-gnucash-c-*-
   gnucash.c -- gnucash startup function.
   
   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.
                                                                   
   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.
                                                                   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
   
   (C) 1998 Rob Browning <rlb@cs.utexas.edu>
*/

#define _GNU_SOURCE
#include <stdio.h>
#include <assert.h>

#include <guile/gh.h>

#include "gnucash.h"

static const char *
scan_argv_for_string_arg(const char *target, const int argc, char *argv[])
{
  const char *result = NULL;
  
  int i;
  for(i = 0; (!result) && (i < argc); i++)
  {
    if(strcmp(target, argv[i]) == 0)
    {
      assert((i + 1) < argc); /* FIXME: This needs to cause a usage failure */
      result = argv[i + 1];
    }
  }
  return result;
}

/* Because swig doesn't declare the prototype anywhere */
void gnucash_swig_init();

static void gnucash_main_helper(int argc, char *argv[])
{  
  const char *startup_file = NULL;
  char *config_file = NULL;
  
  gnucash_swig_init();

  /* Set up the compile-time defaults */

  gh_define("gnc:_config-dir-default_", gh_str02scm(GNC_CONFIG_DIR_DEFAULT));
  gh_define("gnc:_share-dir-default_", gh_str02scm(GNC_SHARE_DIR_DEFAULT));
  gh_define("gnc:_doc-dir-default_", gh_str02scm(GNC_DOC_DIR_DEFAULT));

  /* This has to be done from C because it's the one arg we *have* to
     have (if present) before we start up guile.  It's used to find
     the guile code.  Note that unlike other args the first
     --startup-file always takes effect before any other args. */

  startup_file = scan_argv_for_string_arg("--startup-file", argc, argv);
  
  if(!startup_file)
  {
    startup_file = GNC_SHARE_DIR_DEFAULT "/scm/startup.scm";
  }

  fprintf(stderr, "gnucash: startup-file is %s\n", startup_file);
  
  /* We can't just use gh_eval_str here because --startup-file might have
     double quotes or other funny chars in it.  */
  
  gh_define("gnc:_startup-file-default_", gh_str02scm(startup_file));
  
  if(gh_eval_file(startup_file) == SCM_BOOL_F)
  {
    fprintf(stderr, "gnucash: (load %s) failed\n", startup_file);
    exit(1);
  }
  
  exit(0);
}

void
gnucash_main(int argc, char *argv[])
{
  gh_enter(argc, argv, gnucash_main_helper);
}
